"""
These settings are used by the ``manage.py`` command.

"""
from .settings_test import *  # NOQA


DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'db.sqlite',
    }
}



from django.conf import settings
from django.contrib.staticfiles.testing import StaticLiveServerTestCase
from django.core.urlresolvers import reverse
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support.expected_conditions import _element_if_visible


#determine the WebDriver module. default to Firefox
try:
    webdriver_module = settings.SELENIUM_WEBDRIVER
except AttributeError:
    from selenium.webdriver.firefox import webdriver as webdriver_module


class invisibility_of(object):

    def __init__(self, element):
        self.element = element

    def __call__(self, ignored):
        return not _element_if_visible(self.element)


class CustomWebDriver(webdriver_module.WebDriver):
    """Our own WebDriver with some helpers added"""

    def find_css(self, css_selector):
        """Shortcut to find elements by CSS. Returns either a list or singleton"""
        elems = self.find_elements_by_css_selector(css_selector)
        found = len(elems)
        if found == 1:
            return elems[0]
        elif not elems:
            raise NoSuchElementException(css_selector)
        return elems

    def wait_for_css(self, css_selector, timeout=7):
        """ Shortcut for WebDriverWait"""
        return WebDriverWait(self, timeout).until(lambda driver : driver.find_css(css_selector))



class SeleniumTestCase(StaticLiveServerTestCase):
    """
    A base test case for Selenium, providing hepler methods for generating
    clients and logging in profiles.
    """
    def open(self, url):
        self.wd.get("%s%s" % (self.live_server_url, url))

    def login(self):
        self.open(reverse('admin:index'))

        # Selenium knows it has to wait for page loads (except for AJAX requests)
        # so we don't need to do anything about that, and can just
        # call find_css. Since we can chain methods, we can
        # call the built-in send_keys method right away to change the
        # value of the field
        self.wd.find_css('#id_username').send_keys("admin")
        # for the password, we can now just call find_css since we know the page
        # has been rendered
        self.wd.find_css("#id_password").send_keys('secret')
        # You're not limited to CSS selectors only, check
        # http://seleniumhq.org/docs/03_webdriver.html for
        # a more compreehensive documentation.
        self.wd.find_element_by_xpath('//input[@type="submit"]').click()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TestModel',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('selection', models.CharField(blank=True, max_length=20, verbose_name='Selection', choices=[('', 'Empty'), ('hors', 'Horse'), ('bear', 'Bear'), ('octopus', 'Octopus')])),
                ('horse', models.CharField(max_length=20, blank=True)),
                ('bear', models.CharField(max_length=20, blank=True)),
                ('octopus', models.CharField(max_length=20, blank=True)),
            ],
        ),
    ]



from __future__ import unicode_literals

from django.utils.encoding import python_2_unicode_compatible
from django.db import models


SELECTION_CHOICES = (
    ('', 'Empty'),
    ('horse', 'Horse'),
    ('bear', 'Bear'),
    ('octopus', 'Octopus'),
)


SET_CHOICES = (
    ('', 'Empty'),
    ('set1', '1'),
    ('set2', '2'),
    ('set3', '3'),
)


@python_2_unicode_compatible
class TestModelSingle(models.Model):
    selection = models.CharField('Selection', max_length=20, blank=True, choices=SELECTION_CHOICES)
    horse = models.CharField(max_length=20, blank=True, )
    bear = models.CharField(max_length=20, blank=True, )
    octopus = models.CharField(max_length=20, blank=True, )

    def __str__(self):
        return "Single Stash Test Model: %s" % self.selection


@python_2_unicode_compatible
class TestModelAdvanced(models.Model):
    set = models.CharField('Selection', max_length=20, blank=True, choices=SET_CHOICES)
    set1_1 = models.CharField(max_length=20, blank=True, )
    set2_1 = models.CharField(max_length=20, blank=True, )
    set2_2 = models.CharField(max_length=20, blank=True, )
    set2_3 = models.CharField(max_length=20, blank=True, )
    set3_1 = models.CharField(max_length=20, blank=True, )

    def __str__(self):
        return "Test Model: %s" % self.set


@python_2_unicode_compatible
class TestInlineModel(models.Model):
    parent = models.ForeignKey(TestModelAdvanced)
    title = models.CharField(max_length=20, blank=True, )

    def __str__(self):
        return "A Simple Inline Model: %s" % self.title


@python_2_unicode_compatible
class TestModelInInlineModel(models.Model):
    title = models.CharField(max_length=20, blank=True, )

    def __str__(self):
        return "Has Stash In Inline Model: %s" % self.title


@python_2_unicode_compatible
class TestInlineModelSingle(models.Model):
    parent = models.ForeignKey(TestModelInInlineModel)
    selection = models.CharField('Selection', max_length=20, blank=True, choices=SELECTION_CHOICES)
    horse = models.CharField(max_length=20, blank=True, )
    bear = models.CharField(max_length=20, blank=True, )
    octopus = models.CharField(max_length=20, blank=True, )

    def __str__(self):
        return "Inline Stash Model: %s" % self.selection



from django.contrib import admin

from formfieldstash.admin import FormFieldStashMixin
from .models import TestModelSingle, TestModelAdvanced, TestModelInInlineModel, TestInlineModelSingle, \
    TestInlineModel


class TestModelAdmin(FormFieldStashMixin, admin.ModelAdmin):
    single_formfield_stash = ('selection', )

admin.site.register(TestModelSingle, TestModelAdmin)


class TestInlineModelInline(admin.StackedInline):
    model = TestInlineModel


ADVANCED_STASH = {
    'set': {
        'set1': ('set1_1', '#testinlinemodel_set-group', ),
        'set2': ('set2_1', 'set2_2', 'set2_3', ),
        'set3': ('set3_1', 'set2_1', ),
    },
}


class TestModelAdvancedAdmin(FormFieldStashMixin, admin.ModelAdmin):
    inlines = [TestInlineModelInline, ]
    formfield_stash = ADVANCED_STASH

admin.site.register(TestModelAdvanced, TestModelAdvancedAdmin)


class TestInlineModelSingleInline(FormFieldStashMixin, admin.StackedInline):
    model = TestInlineModelSingle
    single_formfield_stash = ('selection', )


class TestModelInInlineModelAdmin(FormFieldStashMixin, admin.ModelAdmin):
    inlines = [TestInlineModelSingleInline, ]

admin.site.register(TestModelInInlineModel, TestModelInInlineModelAdmin)



"""URLs to run the tests."""
from django.conf import settings
from django.conf.urls import include, url
from django.contrib import admin


admin.autodiscover()

urlpatterns = [
    url(r'^admin/', include(admin.site.urls)),
]

if settings.DEBUG:
    urlpatterns += [
        url(r'^media/(?P<path>.*)$', 'django.views.static.serve', {'document_root': settings.MEDIA_ROOT}),
    ]



from django.forms import widgets


class FormFieldStashMixin(object):

    def formfield_for_dbfield(self, db_field, *args, **kwargs):
        field = super(FormFieldStashMixin, self).formfield_for_dbfield(db_field, *args, **kwargs)
        if getattr(self, 'single_formfield_stash', None):
            for stash_field in self.single_formfield_stash:
                if db_field.name == stash_field:
                    field.widget.attrs['data-formfield-stash'] = "true"
                    field.widget.attrs['data-original-field'] = db_field.name
        if getattr(self, 'formfield_stash', None):
            for stash_field, fields in self.formfield_stash.items():
                if db_field.name == stash_field:
                    field.widget.attrs['data-formfield-stash'] = "true"
                    field.widget.attrs['data-original-field'] = db_field.name
                    for choice, show_fields in fields.items():
                        field.widget.attrs['data-formfield-stash-%s' % choice] = (
                            ','.join(show_fields)
                        )
        return field

    def save_model(self, request, obj, form, change):
        # TODO: implement? design decision! do it on the model!?
        if getattr(self, 'single_formfield_stash', None):
            for stash_field in self.single_formfield_stash:
                for choice, display_choice in form.fields[stash_field].choices:
                    pass
        super(FormFieldStashMixin, self).save_model(request, obj, form, change)

    @property
    def media(self):
        js = (
            'formfield_stash/formfield_stash.js',
        )
        css = {
            'all': ('formfield_stash/adjust-divers.css', ),
        }
        original_media = super(FormFieldStashMixin, self).media
        return original_media + widgets.Media(css=css, js=js)



# -*- coding: utf-8 -*-
from django.core.urlresolvers import reverse
from selenium.webdriver.support.expected_conditions import visibility_of
from selenium.webdriver.support.wait import WebDriverWait

from formfieldstash.tests.utils.django_utils import create_superuser
from formfieldstash.tests.utils.selenium_utils import SeleniumTestCase, CustomWebDriver, \
    invisibility_of
from formfieldstash.tests.test_app.models import TestModelSingle, TestModelAdvanced


class FormFieldStashAdminTests(SeleniumTestCase):
    def setUp(self):
        self.single_empty = TestModelSingle()
        self.single_empty.save()
        self.single = TestModelSingle(**{'selection': 'octopus', })
        self.single.save()
        self.advanced_empty = TestModelAdvanced()
        self.advanced_empty.save()
        self.advanced = TestModelAdvanced(**{'set': 'set1', })
        self.advanced.save()
        self.superuser = create_superuser()
        # Instantiating the WebDriver will load your browser
        self.wd = CustomWebDriver()

    def tearDown(self):
        self.wd.quit()

    def test_app_index_get(self):
        self.login()
        self.open(reverse('admin:index'))
        self.wd.find_css(".app-test_app")

    def test_single_stash_empty(self):
        self.login()
        self.open(reverse('admin:test_app_testmodelsingle_change', args=[self.single_empty.id]))
        horse = self.wd.find_css("div.field-horse")
        # why wait? widget init delays initialization for 20ms, for other widgets to initialize.
        wait = WebDriverWait(self.wd, 1)
        wait.until(invisibility_of(horse))
        # self.assertFalse(horse.is_displayed())
        bear = self.wd.find_css("div.field-bear")
        wait.until(invisibility_of(bear))
        # self.assertFalse(bear.is_displayed())
        octo = self.wd.find_css("div.field-octopus")
        wait.until(invisibility_of(octo))
        # self.assertFalse(octo.is_displayed())

    def test_single_stash(self):
        self.login()
        self.open(reverse('admin:test_app_testmodelsingle_change', args=[self.single.id]))
        horse = self.wd.find_css("div.field-horse")
        wait = WebDriverWait(self.wd, 1)
        wait.until(invisibility_of(horse))
        # self.assertFalse(horse.is_displayed())
        bear = self.wd.find_css("div.field-bear")
        wait.until(invisibility_of(bear))
        # self.assertFalse(bear.is_displayed())
        octo = self.wd.find_css("div.field-octopus")
        wait.until(visibility_of(octo))
        # self.assertTrue(octo.is_displayed())
        # change select value
        self.wd.find_css("div.field-selection select > option[value=horse]").click()
        horse = self.wd.find_css("div.field-horse")
        self.assertTrue(horse.is_displayed())
        octo = self.wd.find_css("div.field-octopus")
        self.assertFalse(octo.is_displayed())

    def test_multi_stash_empty(self):
        self.login()
        self.open(reverse('admin:test_app_testmodeladvanced_change', args=[self.advanced_empty.id]))
        inline = self.wd.find_css("#testinlinemodel_set-group")
        wait = WebDriverWait(self.wd, 1)
        wait.until(invisibility_of(inline))
        # self.assertFalse(inline.is_displayed())
        f11 = self.wd.find_css("div.field-set1_1")
        wait.until(invisibility_of(f11))
        # self.assertFalse(f11.is_displayed())
        f31 = self.wd.find_css("div.field-set3_1")
        wait.until(invisibility_of(f31))
        # self.assertFalse(f31.is_displayed())

    def test_multi_stash(self):
        self.login()
        self.open(reverse('admin:test_app_testmodeladvanced_change', args=[self.advanced.id]))
        inline = self.wd.find_css("#testinlinemodel_set-group")
        wait = WebDriverWait(self.wd, 1)
        wait.until(visibility_of(inline))
        # self.assertTrue(inline.is_displayed())
        f11 = self.wd.find_css("div.field-set1_1")
        wait.until(visibility_of(f11))
        # self.assertTrue(f11.is_displayed())
        f31 = self.wd.find_css("div.field-set3_1")
        wait.until(invisibility_of(f31))
        # self.assertFalse(f31.is_displayed())




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('test_app', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='TestInlineModel',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=20, blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='TestInlineModelSingle',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('selection', models.CharField(blank=True, max_length=20, verbose_name='Selection', choices=[('', 'Empty'), ('horse', 'Horse'), ('bear', 'Bear'), ('octopus', 'Octopus')])),
                ('horse', models.CharField(max_length=20, blank=True)),
                ('bear', models.CharField(max_length=20, blank=True)),
                ('octopus', models.CharField(max_length=20, blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='TestModelAdvanced',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('set', models.CharField(blank=True, max_length=20, verbose_name='Selection', choices=[('', 'Empty'), ('set1', '1'), ('set2', '2'), ('set3', '3')])),
                ('set1_1', models.CharField(max_length=20, blank=True)),
                ('set2_1', models.CharField(max_length=20, blank=True)),
                ('set2_2', models.CharField(max_length=20, blank=True)),
                ('set2_3', models.CharField(max_length=20, blank=True)),
                ('set3_1', models.CharField(max_length=20, blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='TestModelInInlineModel',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=20, blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='TestModelSingle',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('selection', models.CharField(blank=True, max_length=20, verbose_name='Selection', choices=[('', 'Empty'), ('horse', 'Horse'), ('bear', 'Bear'), ('octopus', 'Octopus')])),
                ('horse', models.CharField(max_length=20, blank=True)),
                ('bear', models.CharField(max_length=20, blank=True)),
                ('octopus', models.CharField(max_length=20, blank=True)),
            ],
        ),
        migrations.DeleteModel(
            name='TestModel',
        ),
        migrations.AddField(
            model_name='testinlinemodelsingle',
            name='parent',
            field=models.ForeignKey(to='test_app.TestModelInInlineModel'),
        ),
        migrations.AddField(
            model_name='testinlinemodel',
            name='parent',
            field=models.ForeignKey(to='test_app.TestModelAdvanced'),
        ),
    ]
